import { AcDataGridHeader } from './ac-datagridheader';
import { AcDataGridItem } from './ac-datagriditem';
export var Direction;
(function (Direction) {
    Direction[Direction["Ascending"] = 0] = "Ascending";
    Direction[Direction["Descending"] = 1] = "Descending";
})(Direction || (Direction = {}));
/**
 * The controlling class for Grid applications.
 *
 * This class takes an array of elements and handles:
 * - visible headers, managed by @Hidden() decorator
 * - create header titles, managed by @Display() decorator
 * - sorting
 * - filtering
 * - count total rows
 * - paging
 */
var AcDataGridModel = (function () {
    function AcDataGridModel(items, typeInstance, pageSize) {
        if (pageSize === void 0) { pageSize = 10; }
        this.searchValue = {};
        this.currentPageIndex = 1;
        this._items = items;
        this.pageSize = pageSize;
        if (typeInstance) {
            // make header from decorators, omit if null
            this.createHeadersForType(typeInstance);
        }
    }
    Object.defineProperty(AcDataGridModel.prototype, "totalRows", {
        get: function () {
            return this._items.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "totalFilteredRows", {
        get: function () {
            return (this.itemsFiltered) ? this.itemsFiltered.length : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "currentRowStart", {
        get: function () {
            return this.totalRows > this.pageSize
                ? (this.startRow + 1)
                : this.totalRows === 0 ? 0 : 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "currentRowEnd", {
        get: function () {
            return (this.startRow + this.pageSize) < this.totalRows
                ? (this.startRow + this.pageSize)
                : this.totalRows;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "startRow", {
        get: function () {
            if (this.currentPageIndex === 0) {
                return 0;
            }
            return (this.currentPageIndex - 1) * this.pageSize;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "maxPageIndex", {
        get: function () {
            var index = Math.ceil(this.totalFilteredRows / this.pageSize);
            return index;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "items", {
        get: function () {
            return this._items;
        },
        set: function (value) {
            this._items = value;
        },
        enumerable: true,
        configurable: true
    });
    AcDataGridModel.prototype.getItemSorted = function (sortColumn, sortDirection) {
        if (sortDirection === Direction.Ascending) {
            return this.items.sort(function (a, b) { return a[sortColumn] > b[sortColumn] ? 1 : -1; });
        }
        else {
            return this.items.sort(function (a, b) { return a[sortColumn] < b[sortColumn] ? 1 : -1; });
        }
    };
    Object.defineProperty(AcDataGridModel.prototype, "itemsFiltered", {
        get: function () {
            var _this = this;
            return this.items.filter(function (item) { return Object.same(_this.searchValue, item); });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "itemsOnCurrentPage", {
        get: function () {
            return this.itemsFiltered.slice(this.startRow, this.startRow + this.pageSize);
        },
        enumerable: true,
        configurable: true
    });
    // The view can get col by col filtered for valid headers
    AcDataGridModel.prototype.columnsOfItemValues = function (item) {
        // we return all if no headers
        var columns = new Array();
        if (!this._headers) {
            for (var prop in item) {
                columns.push(item[prop]);
            }
        }
        else {
            this.headers
                .forEach(function (e, idx) { return columns.push(item[e.prop]); });
        }
        return columns;
    };
    AcDataGridModel.prototype.columnsOfItem = function (item) {
        // we return all if no headers
        var columns = new Array();
        var currentHeaders = this.headers || Object.keys(item).map(function (h) { return new AcDataGridHeader(h, null, h, false); });
        currentHeaders
            .forEach(function (h, idx) {
            var e = new AcDataGridItem();
            e.value = item[h.prop];
            e.prop = h.prop;
            var hasPipe = e["__hasPipe__" + h.prop];
            if (hasPipe) {
                e.pipeToken = hasPipe;
            }
            columns.push(e);
        });
        return columns;
    };
    AcDataGridModel.prototype.sortColumn = function (colName, dir) {
        this.items.sort(function (a, b) { return dir === 'desc' ? (a[colName] > b[colName] ? 1 : -1) : (a[colName] > b[colName] ? -1 : 1); });
    };
    Object.defineProperty(AcDataGridModel.prototype, "headers", {
        get: function () {
            return this._headers.filter(function (h) { return !h.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AcDataGridModel.prototype, "headersNotVisible", {
        get: function () {
            return this._headers.filter(function (h) { return h.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    AcDataGridModel.prototype.removeColumn = function (colname) {
        var col = this._headers.find(function (h) { return h.prop === colname; });
        if (col) {
            col.hidden = true;
        }
    };
    AcDataGridModel.prototype.addColumn = function (colname) {
        var col = this._headers.find(function (h) { return h.prop === colname; });
        if (col) {
            col.hidden = false;
        }
    };
    AcDataGridModel.prototype.createHeadersForType = function (type) {
        // assume simple object structure, iterating an array of viewmodels
        // has at least one row, so we can read the headers
        // first we read the properties
        this._headers = new Array();
        for (var p in type) {
            // either propname or decorator name
            var propName = type["__displayName__" + p] || p;
            var propDesc = type["__displayDesc__" + p] || p;
            // check if hidden, show if no hidden decorator
            var isHidden = type["__isHidden__" + p] || false;
            this._headers.push(new AcDataGridHeader(propName, propDesc, p, isHidden));
        }
    };
    return AcDataGridModel;
}());
export { AcDataGridModel };

//# sourceMappingURL=ac-datagridmodel.js.map
